#ifndef PACKAGEMANAGER_H
#define PACKAGEMANAGER_H

#include "libflyadminpackage_global.h"

#include <QObject>
#include <QSharedPointer>

namespace FlyAdmin
{

class Package;

class FLYADMINPACKAGE_EXPORT PackageManager : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(PackageManager)

public:
    using PackagePtr = QSharedPointer<Package>;

    static PackageManager* instance();
    ~PackageManager();

    /*!
     * \brief Check if package is installed.
     *
     * It uses dpkg-query to get package info:
     * dpkg-query --showformat='${db:Status-Abbrev}${binary:Package}\\n' --show package
     * \n Can be used to check complex cases like:
     * \code
     * isPackageInstalled("postgresql-[0123456789]*", "ii\\spostgresql-(\\d+\\.\\d+)");
     * \endcode
     *
     * \param package Can be used with patterns like package*
     * \param installedRegex Each line is compared with this regex
     * \return true if each line is matched with installedRegex
     */
    Q_INVOKABLE static bool isPackageInstalled(const QString &package, const QString &installedRegex = QLatin1String("ii\\s%1"));

    /*!
     * \brief Open synaptic util to install or uninstall chosen package.
     *
     * If package found it will be selected and filtered among other packages.
     * User only need to click Accept button.
     *
     * \param package to install or uninstall
     * \param uninstall flag of uninstalling
     * \return false if fail to run synaptic or save filter files
     */
    Q_INVOKABLE static bool runSynapticWithPackageSelected(const QString &package, bool uninstall = false);

    /*!
     * \brief Not ready yet..
     */
    PackagePtr getPackage(const QString &package) const;

    /*!
     * \brief Returns a list of versions of the specified packages
     *
     * It uses dpkg-query to get package info:
     * dpkg-query --showformat='${Version}\n' --show nameTemplate
     *
     * \param nameTemplate Can be used with patterns like package*
     * \return list of packages versions or empty list
     */
    Q_INVOKABLE static const QStringList getInstalledPackagesVersions(const QString &nameTemplate);

private:
    explicit PackageManager(QObject *parent = nullptr);

    static PackageManager* m_pinstance;
};

}

#endif // PACKAGEMANAGER_H
